from .mg import OptionHelper

class Layout(OptionHelper):
    """Layout options for the MetricsGraphics API.

        https://github.com/mozilla/metrics-graphics/wiki/Layout
    """
    _allowed_layout = [
        "buffer",
        "bottom",
        "height",
        "left",
        "right",
        "small_height_threshold",
        "small_width_threshold",
        "top",
        "width"
    ]

    def set_margin(self, top=40, bottom=30, left=50, right=10, buffer_size=8):
        """Set margin of the chart.

            Args:
                top (int): size of top margin in pixels.
                bottom (int): size of bottom margin in pixels.
                left (int): size of left margin in pixels.
                right (int): size of right margin in pixels.
                buffer_size (int): buffer size in pixels between the chart and margins.

        """
        self.set_integer("top", top)
        self.set_integer("bottom", bottom)
        self.set_integer("left", left)
        self.set_integer("right", right)
        self.set_integer("buffer", buffer_size)

    def set_size(self, height=220, width=350,
                 height_threshold=120,
                 width_threshold=160):
        """Set the size of the chart.

            Args:
                height (int): height in pixels.
                width (int): width in pixels.
                height_threshold (int): height threshold in pixels
                width_threshold (int): width threshold in pixesls

        """
        self.set_integer("height", height)
        self.set_integer("width", width)
        self.set_integer("small_height_threshold", height_threshold)
        self.set_integer("small_width_threshold", width_threshold)

    def get(self):
        """Get layout options."""
        return {k:v for k,v in list(self.options.items()) if k in self._allowed_layout}


from .react_template import ReactComponent
from collections import OrderedDict
from flask import jsonify

def register_layouts(layouts, app, url="/api/props/", brand="Pyxley"):
    """ register UILayout with the flask app

        create a function that will send props for each UILayout

        Args:
            layouts (dict): dict of UILayout objects by name
            app (object): flask app
            url (string): address of props; default is /api/props/
    """
    def props(name):
        if name not in layouts:
            # cast as list for python3
            name = list(layouts.keys())[0]
        return jsonify({"layouts": layouts[name]["layout"]})

    def apps():
        paths = []
        for i, k in enumerate(layouts.keys()):
            if i == 0:
                paths.append({
                    "path": "/",
                    "label": layouts[k].get("title", k)
                })

            paths.append({
                "path": "/"+k,
                "label": layouts[k].get("title", k)
            })

        return jsonify({"brand": brand, "navlinks": paths})

    app.add_url_rule(url+"<string:name>/", view_func=props)
    app.add_url_rule(url, view_func=apps)

class UIComponent(object):
    """Base React UI component.

        Every UI component is derived from this component. Both
        charts and filters inherit these methods. This class allows
        the chart to register the endpoint with the flask application.

        Args:
            component_type (str): Type of component to build.
            options (dict): options specific to the component type.
            route_func (function): endpoint function.

    """
    def __init__(self, component_type, options, route_func):
        """"""
        self.params = {
            "type": component_type,
            "options": options
        }
        self.route_func = route_func

    def register_route(self, app):
        """Register the api route function with the app."""
        if "url" not in self.params["options"]:
            raise Exception("Component does not have a URL property")

        if not hasattr(self.route_func, "__call__"):
            raise Exception("No app route function supplied")

        app.add_url_rule(self.params["options"]["url"],
            self.params["options"]["url"],
            self.route_func)


class SimpleComponent(object):
    """Simple class for rendering a single component.

        Rather than requiring a collection of filters and charts, this
        function will render a single react component with props
        passed as a dictionary.

        Args:
            layout (str): Type of react component to create.
            src_file (str): javascript file containing the component.
            component_id (str): html element id.
            props (dict): props for the component.
    """
    def __init__(self, layout, src_file, component_id, props):
        self.layout = layout
        self.src_file = src_file
        self.component_id = component_id
        self.props = props

    def render(self, path):
        """Render the component to a javascript file."""
        return ReactComponent(
            self.layout,
            self.src_file,
            self.component_id,
            props=self.props,
            static_path=path)

class UILayout(object):
    """Simple UI layout.

        This class handles the construction of a single javascript file
        based on the inputs provided. It aggregates the properties specified
        over the different charts and filters and creates a single set of
        props.

        This relies on the creation of a single React component that is comprised
        of charts and filters. An example can be found at
        https://github.com/stitchfix/pyxleyJS/blob/master/src/layouts.js

        Args:
            layout (str): name of react component to render.
            src_file (str): location of javascript containing the component.
            component_id (str): html element id.

    """
    def __init__(self, layout, component_id="component_id", src_file='pyxley'):

        self.layout = layout
        self.src_file = src_file
        self.component_id = component_id
        self.filters = OrderedDict()
        self.charts = []

    def add_filter(self, component, filter_group="pyxley-filter"):
        """Add a filter to the layout."""
        if getattr(component, "name") != "Filter":
            raise Exception("Component is not an instance of Filter")
        if filter_group not in self.filters:
            self.filters[filter_group] = []
        self.filters[filter_group].append(component)

    def add_chart(self, component):
        """Add a chart to the layout."""
        if getattr(component, "name") != "Chart":
            raise Exception("Component is not an instance of Chart")
        self.charts.append(component)

    def build_props(self):
        """Build the props dictionary."""
        props = {}
        if self.filters:
            props["filters"] = {}
            for grp in self.filters:
                props["filters"][grp] = [f.params for f in self.filters[grp]]
        if self.charts:
            props["charts"] = [c.params for c in self.charts]

        props["type"] = self.layout
        return props

    def assign_routes(self, app):
        """Register routes with the app."""
        for grp in self.filters:
            for f in self.filters[grp]:
                if f.route_func:
                    f.register_route(app)

        for c in self.charts:
            if c.route_func:
                c.register_route(app)

    def render_layout(self, app, path, alias=None):
        """Write to javascript."""
        self.assign_routes(app)
        return ReactComponent(
            self.layout,
            self.src_file,
            self.component_id,
            props=self.build_props(),
            static_path=path,
            alias=alias)



from jinja2 import Template
import json

class ReactTemplate(object):
    """Base React class.

        This class handles the basic transformation of the React jsx code.
        It takes a dictionary of options, renders the template, and
        writes the transformed javascript file.

        Args:
            template (str): jinja2 template for the react jsx code.
            template_args (dict): options to be populated in the template.
            path (str): path of the file to create.
    """
    def __init__(self, template, template_args, path):
        self.template = template
        self.args = template_args
        self.path = path

    def write_to_file(self, s):
        """
        """
        f = open(self.path, 'w')
        f.write(s)
        f.close()

    def to_js(self):
        """
        """
        t = Template(self.template)
        js = t.render(**self.args)
        self.write_to_file(js)

def format_props(props, prop_template="{{k}} = { {{v}} }", delim="\n"):
    """ Formats props for the React template.

        Args:
            props (dict): properties to be written to the template.

        Returns:
            Two lists, one containing variable names and the other
            containing a list of props to be fed to the React template.

    """
    vars_ = []
    props_ = []
    for k, v in list(props.items()):
        vars_.append(Template("var {{k}} = {{v}};").render(k=k,v=json.dumps(v)))
        props_.append(Template(prop_template).render(k=k, v=k))
    return "\n".join(vars_), delim.join(props_)


class ReactComponent(ReactTemplate):
    """Class to generate javascript from parameters.

        This class create a javascript file by compiling a JSX string containing
        React components. A single parent component is rendered and sub-components
        are passed into the component as props.

        Args:
            name (str): name of the ReactComponent to create.
            path (str): location of the javascript file containing the component.
            element_id (str): html element id.
            props (dict): properties of the react component to render.
            static_path (str): output file destination.
    """
    _react = (
    """
    import React from 'react';
    import ReactDOM from 'react-dom';
    import { {{name}} as Component} from '{{path}}';
    {{vars}}
    ReactDOM.render(
        <Component
        {{props}} />,
        document.getElementById("{{id}}")
    );
    """)

    _react_export = (
    """
    import React from 'react';
    import ReactDOM from 'react-dom';
    import { {{name}} as Component} from '{{path}}';

    {{vars}}

    class {{alias}} extends React.Component {
        constructor(props) {
            super(props);
        }

        render() {
            return (
                <Component { ...this.props } />
            );
        }
    }
    {{alias}}.defaultProps = {
        {{props}}
    };
    export default {{alias}};

    """)
    def __init__(self, name, path, element_id,
        props={}, static_path="", alias=None):

        _template = self._react
        prop_template = "{{k}} = { {{v}} }"
        delim = "\n"
        if alias:
            prop_template = "{{k}} : {{v}}"
            delim = ",\n"
            _template = self._react_export

        vars_, props_ = format_props(props,
            prop_template=prop_template, delim=delim)

        params = {
            "name": name,
            "alias": alias,
            "path": path,
            "vars": vars_,
            "props": props_,
            "id": element_id
        }
        for k, v in list(props.items()):
            params[k] = v

        super(ReactComponent, self).__init__(
            _template, params, static_path)
        self.to_js()




from __future__ import print_function

import os
import sys
import json

from ..react_template import ReactTemplate
from .npm import run
from subprocess import check_call

class Webpack(object):
    webpack_template = """
        var path = require("path");
        var webpack = require("webpack");

        module.exports = {
          entry: [
              '{{include_path}}/{{entry_point}}'
          ],
          output: {
            path: path.join(__dirname,'{{output_path}}'),
            sourceMapFilename: '{{output_name}}.map',
            filename: '{{output_name}}.js'
          },
          resolve: {
              extensions: ['', '.js', '.scss'],
              alias: {
                  react: path.resolve('./node_modules/react')
              }
          },

          module: {
            loaders: [
              {
                test: /\.js?$/,
                include: path.join(__dirname,'{{include_path}}'),
                loader: ['babel-loader'],
                exclude: /node_modules/,
                query: {
                    presets: ['es2015', 'react']
                }
              },
              {
                test: /\.scss$/,
                loaders: ['style', 'css', 'sass']
              },
              {
                test: /\.json$/,
                loader: 'json'
              }
            ]
        },
        plugins: [
                new webpack.ProvidePlugin({
                  $: "jquery",
                  jQuery: "jquery",
                  MG: "metrics-graphics",
                  Datamap: "datamaps",
                  Plotly: "plotly.js",
                  nv: "nvd3"
                })
            ]
        };
    """

    def __init__(self, repo_root):
        self.repo_root = repo_root

    def create_webpack_config(self, main_js, include_path,
        output_name, output_path):

        options = {
            "entry_point": main_js,
            "include_path": include_path,
            "output_name": output_name,
            "output_path": output_path
        }
        rt = ReactTemplate(self.webpack_template,
            options, self.repo_root+"/webpack.config.js")
        rt.to_js()


    def run(self):

        # check repo root for a package.json file
        if not os.path.isfile(self.repo_root+"/webpack.config.js"):
            print("Creating webpack.config.js...", file=sys.stderr)
            self.create_webpack_config("index.js", ".", "bundle", ".")

        try:
            run(["npm", "run-script", "build"], cwd=self.repo_root)
        except Exception as e:
            print("Failed to run `npm run-script build`: %s" % e, file=sys.stderr)
            print("Please resolve javascript errors", file=sys.stderr)
            print("Try running with --display-error-details")
            raise



from __future__ import print_function

import os
import sys
import json

from ..react_template import ReactTemplate
from subprocess import check_call

def run(cmd, *args, **kwargs):
    """"""
    kwargs["shell"] = (sys.platform == "win32")
    return check_call(cmd, *args, **kwargs)

class NPM(object):
    package_template = ("""
    {
      "name": "{{name}}",
      "version": "{{version}}",
      "description": "",
      "main": "{{main}}",
      "scripts": {{scripts}},
      "author": "",
      "license": "MIT",
      "devDependencies": {{dev_dependencies}},
      "dependencies": {{dependencies}}
    }
    """)

    dev_dependencies = {
        "babel-core": "^6.5.0",
        "babel-loader": "^6.2.2",
        "babel-preset-es2015": "^6.5.0",
        "babel-preset-react": "^6.5.0",
        "bower": "^1.7.7",
        "css-loader": "^0.23.1",
        "d3": "^3.5.16",
        "jquery": "^2.2.0",
        "node-sass": "^3.3.3",
        "nvd3": "^1.8.2",
        "datamaps": "^0.4.4",
        "metrics-graphics": "^2.8.0",
        "datatables": "^1.10.7",
        "plotly.js": "^1.5.2",
        "react": "^0.14.7",
        "react-bootstrap": "^0.28.3",
        "react-dom": "^0.14.7",
        "react-router": "^2.0.0",
        "sass-loader": "^3.1.2",
        "style-loader": "^0.13.0",
        "webpack": "^1.12.13",
        "webpack-dev-server": "^1.14.1",
        "pyxley": "^0.0.6"
    }
    def __init__(self, repo_root):
        self.repo_root = repo_root

    def create_package_json(self,
            name="PyxleyApp",
            version="0.0.1",
            main="index.js",
            dependencies={},
            scripts={}):
        """
        """

        if "build" not in scripts:
            _build = "node_modules/.bin/webpack"
            scripts["build"] = _build

        options = {
            "name": name,
            "version": version,
            "main": main,
            "scripts": json.dumps(scripts),
            "dependencies": json.dumps(dependencies),
            "dev_dependencies": json.dumps(self.dev_dependencies)
        }

        rt = ReactTemplate(self.package_template,
            options, self.repo_root+"/package.json")
        rt.to_js()

    def run(self):
        # check repo root for a package.json file
        if not os.path.isfile(self.repo_root+"/package.json"):
            print("Creating package.json...", file=sys.stderr)
            self.create_package_json()


        cmds = ["npm", "install", "--progress-false"]
        try:
            run(cmds, cwd=self.repo_root)
        except OSError as e:
            print("Failed to run `npm install`: %s" % e, file=sys.stderr)
            print("npm is required to build a pyxley app", file=sys.stderr)
            raise



from .react_template import ReactTemplate

def get_property(x, prop):
    props = []
    for k in x:
        if prop in x[k]:
            props.append((k, x[k][prop]))
    return props

class ReactRouter(ReactTemplate):
    """Class to generate javascript for react-router.

        This class creates a jsx file for react-router and assembles
        the jsx components for several pages

        Args:
            components (dict): routes and files by component name
            element_id (str): html element id.
            static_path (str): output file destination.
    """
    _template = (
    """
    import React from 'react';
    import ReactDOM from 'react-dom';
    import { Router, Route, Link, browserHistory } from 'react-router';

    {% for component, filename in components %}
    import {{component}} from './{{filename}}';
    {% endfor %}

    ReactDOM.render(
      <Router history={ browserHistory }>
      {% for component, route in routes %}
      <Route path='{{route}}' component={ {{component}} } />
      {% endfor %}
      </Router>,
      document.getElementById("{{id}}")
    );

    """)

    def __init__(self, components, element_id, static_path=""):
        routes = get_property(components, "route")
        imports = get_property(components, "filename")

        params = {
            "components": imports,
            "routes": routes,
            "id": element_id
        }

        super(ReactRouter, self).__init__(
            self._template, params, static_path)
        self.to_js()



from .mg import MG
from flask import jsonify, request

class LineChart(MG):
    """Wrapper for MetricsGraphics Line plot.

        https://github.com/mozilla/metrics-graphics/wiki/Chart-Types
        The user must supply a Figure object. When a route_func
        is supplied, the dataframe and other options are ignored
        in favor of the endpoint function.

        Args:
            df (dataframe): input data.
            figure (mg.Figure): metricsgraphics figure object.
            x (str): name of the column corresponding to the x-axis.
            y (list(str)): name of the columns corresponding to the y-axis.
            title (str): title of the figure.
            description (str): description of the figure.
            init_params (dict): parameters used to initialize the figure.
            timeseries (bool): inidicates whether the plot is a timeseries.
            route_func (function): endpoint function.

    """
    def __init__(self, df, figure, x, y, title="Line Chart",
        description="Line Chart", init_params={}, timeseries=False,
        route_func=None):

        self.plot_opts = {
            "title": title,
            "description": description,
            "target": "#"+figure.chart_id,
            "x_accessor": "x",
            "y_accessor": "y",
            "init_params": init_params
        }
        for k, v in list(figure.get().items()):
            self.plot_opts[k] = v

        if not route_func:
            def get_data():
                args = {}
                for c in init_params:
                    if request.args.get(c):
                        args[c] = request.args[c]
                    else:
                        args[c] = init_params[c]
                return jsonify(LineChart.to_json(
                        self.apply_filters(df, args),
                        x,
                        y,
                        timeseries=timeseries
                    ))
            route_func = get_data

        super(LineChart, self).__init__(figure.chart_id, figure.url, self.plot_opts, route_func)

    @staticmethod
    def to_json(df, x, y, timeseries=False):
        """Format output for json response."""
        values = {k: [] for k in y}
        for i, row in df.iterrows():
            for yy in y:
                values[yy].append({
                    "x": row[x],
                    "y": row[yy]
                    })
        return {"result":  [values[k] for k in y], "date": timeseries}



from ..charts import Chart

class NVD3(Chart):
    """Base class for NVD3 Charts.

        This class is used to create charts from http://nvd3.org/

    """
    def __init__(self, opts, api_route):
        super(NVD3, self).__init__("NVD3Chart", opts, api_route)




from .nvd3 import NVD3
from flask import jsonify, request
import numpy as np

class TwoAxisFocus(NVD3):
    """NVD3 Two Axis Focus

        Wrapper for the cross-filter chart implemented in PyxleyJS based on
        http://nvd3.org/examples/linePlusBar.html. When a route_func is supplied,
        the inputs are ignored in favor of the endpoint function.

        Args:
            values (dict): labels of the pie chart.
            data_source (dataframe): chart data.
            x (str): name of the column for the x-axis.
            y1 (str): name of the column for the primary y-axis.
            y2 (str): name of the column for the secondary y-axis.
            init_params (dict): parameters used to initialize the chart.
            chart_id: html element id.
            url: name of the endpoint to be created.
            colors (list): list of colors to show in the chart.
            auto_scale (str): select auto scale method.
            y1_axis_range (list): range of the primary y-axis.
            y2_axis_range (list): range of the secondary y-axis.
            x_label (str): label of the x-axis.
            y1_label (str): label of the primary axis.
            y2_label (str): label of the secondary axis.
            margin (dict): margins in pixels.
            route_func (function): optional endpoint function.

    """
    _allowed_axes = ["sigma", "minmax"]
    def __init__(self, x, y1, y2, data_source, init_params={},
        chart_id="new_chart", url="/new_chart/", colors=[], auto_scale="sigma",
        y1_axis_range=[], y2_axis_range=[], sigma=3,
        x_label="", y1_label="", y2_label="",
        margin={"top": 30, "right": 60, "bottom": 50, "left": 70},
        route_func=None):


        self.auto_scale = auto_scale if auto_scale in self._allowed_axes else "sigma"

        self.options = {
            "type": "TwoAxisFocus",
            "chartid": chart_id,
            "url": url,
            "colors": colors,
            "init_params": init_params,
            "labels": {
                "xAxis": x_label,
                "yAxis1": y1_label,
                "yAxis2": y2_label
            },
            "margin": margin,
            "type": "TwoAxisFocus"
        }
        if not route_func:
            def get_data():
                args = {}
                for c in init_params:
                    if request.args.get(c):
                        args[c] = request.args[c]
                    else:
                        args[c] = init_params[c]
                return jsonify(self.to_json(
                        self.apply_filters(data_source, args),
                        x, y1, y2, y1_axis_range, y2_axis_range,
                        self.auto_scale, sigma
                    ))
            route_func = get_data

        super(TwoAxisFocus, self).__init__(self.options, route_func)

    @staticmethod
    def get_bounds(y, method="sigma", sigma=3):
        if method == "sigma":
            m_, s_ = y.mean(), y.std()
            l = m_ - sigma*s_
            u = m_ + sigma*s_
        else:
            l = y.min()
            u = y.max()
        return [l, u]

    @staticmethod
    def to_json(df, x, y1, y2, y1_axis_range, y2_axis_range, auto_scale, sigma):
        if df.empty:
            return {
                "data": [],
                "yAxis1": {"lower": 0, "upper": 1},
                "yAxis2": {"lower": 0, "upper": 1}
            }

        if not y1_axis_range:
            bounds1 = TwoAxisFocus.get_bounds(df[y1], method=auto_scale, sigma=sigma)
        else:
            bounds1 = y1_axis_range

        if not y2_axis_range:
            bounds2 = TwoAxisFocus.get_bounds(df[y2], method=auto_scale, sigma=sigma)
        else:
            bounds2 = y2_axis_range

        records = [
            {"key": y1, "values": [], "yAxis": 1, "type": "line"},
            {"key": y2, "values": [], "yAxis": 2, "type": "line"}
        ]

        for n, r in df.iterrows():
            records[0]["values"].append({"x": r[x], "y": r[y1]})
            records[1]["values"].append({"x": r[x], "y": r[y2]})

        return {
            "data": records,
            "yAxis1": {"bounds": bounds1},
            "yAxis2": {"bounds": bounds2}
            }



from ..charts import Chart

class MG(Chart):
    """Base class for MetricsGraphics API.

        This class creates a MetricsGraphics component based on the supplied
        options.
        https://github.com/mozilla/metrics-graphics/wiki/List-of-Options

        Args:
            chart_id (str): html element id.
            url (str): name of the endpoint to create.
            params (dict): parameters and options of the chart.
            api_route (function): endpoint function.
    """
    def __init__(self, chart_id, url, params, api_route):
        opts = {
            "url": url,
            "chart_id": chart_id,
            "params": params
        }
        super(MG, self).__init__("MetricsGraphics", opts, api_route)

class OptionHelper(object):
    """Option Base class for the MetricsGraphics API."""
    def __init__(self):
        self.options = {}

    def set_float(self, option, value):
        """Set a float option.

            Args:
                option (str): name of option.
                value (float): value of the option.

            Raises:
                TypeError: Value must be a float.
        """
        if not isinstance(value, float):
            raise TypeError("Value must be a float")
        self.options[option] = value

    def set_integer(self, option, value):
        """Set an integer option.

            Args:
                option (str): name of option.
                value (int): value of the option.

            Raises:
                ValueError: Value must be an integer.
        """
        try:
            int_value = int(value)
        except ValueError as err:
            print(err.args)

        self.options[option] = value

    def set_boolean(self, option, value):
        """Set a boolean option.

            Args:
                option (str): name of option.
                value (bool): value of the option.

            Raises:
                TypeError: Value must be a boolean.
        """
        if not isinstance(value, bool):
            raise TypeError("%s must be a boolean" % option)

        self.options[option] = str(value).lower()

    def set_string(self, option, value):
        """Set a string option.

            Args:
                option (str): name of option.
                value (str): value of the option.

            Raises:
                TypeError: Value must be a string.
        """
        if not isinstance(value, str):
            raise TypeError("%s must be a string" % option)

        self.options[option] = value





from .nvd3 import NVD3
from flask import jsonify, request
import numpy as np

class PieChart(NVD3):
    """NVD3 Pie Chart

        Wrapper for the NVD3 pie chart implemented in PyxleyJS based on
        http://nvd3.org/examples/pie.html. When a route_func is supplied,
        the inputs are ignored in favor of the endpoint function.

        Args:
            values (dict): labels of the pie chart.
            df (dataframe): chart data.
            init_params (dict): parameters used to initialize the chart.
            chart_id: html element id.
            url: name of the endpoint to be created.
            colors (list): list of colors to show in the chart.
            label_type (str): type of label to display.
            route_func (function): optional endpoint function.

    """
    _allowed = ["key", "value", "percent"]
    def __init__(self, values, df, init_params={},
        chart_id="piechart", url="/piechart/", colors=[],
        label_type="percent", route_func=None):

        if label_type not in self._allowed:
            label_type = "percent"

        self.options = {
            "type": "PieChart",
            "chartid": chart_id,
            "url": url,
            "colors": colors,
            "init_params": init_params,
            "labelType": label_type
        }

        if not route_func:
            def get_data():
                args = {}
                for c in init_params:
                    if request.args.get(c):
                        args[c] = request.args[c]
                    else:
                        args[c] = init_params[c]
                return jsonify(self.to_json(
                            self.apply_filters(df, args),
                            values
                        ))
            route_func = get_data

        super(PieChart, self).__init__(self.options, route_func)

    @staticmethod
    def to_json(df, values):
        """Format output for the json response."""
        records = []
        if df.empty:
            return {"data": []}

        sum_ = float(np.sum([df[c].iloc[0] for c in values]))
        for c in values:
            records.append({
                "label": values[c],
                "value": "%.2f"%np.around(df[c].iloc[0] / sum_, decimals=2)
                })
        return {
            "data" : records
        }



from setuptools import setup

setup(
    name='pyxley',
    version='0.1.0',
    author='Nicholas Kridler',
    author_email='nmkridler@gmail.com',
    license='MIT',
    description='Python tools for building Flask-based web applications',
    packages = ['pyxley', 'pyxley.filters', 'pyxley.charts', 'pyxley.charts.mg',
        'pyxley.charts.datatables', 'pyxley.charts.datamaps',
        'pyxley.charts.nvd3', 'pyxley.utils', 'pyxley.charts.plotly'],
    long_description='Python tools for building Flask-based web applications using React.js',
    url='https://github.com/stitchfix/pyxley',
    keywords=['pyreact', 'flask'],
    classifiers=[
        'Intended Audience :: Developers',
    ],
    install_requires=[
        'flask',
        'pandas'
    ],
    scripts = [
        "bin/pyxapp"
    ],
    package_data = {
        'pyxley': [
            'LICENSE.txt',
            'pyxley/assets/bundle.js',
            'pyxley/assets/templates/index.html',
            'pyxley/assets/css/main.css'
        ]
    },
    include_package_data=True
    # data_files=[
    #     ('pyxley/assets', [
    #         'assets/bundle.js',
    #         'assets/templates/index.html',
    #         'assets/css/main.css'
    #     ])
    # ],

)



from ..ui import UIComponent

class Filter(UIComponent):
    """Base class for Filter UIComponent"""
    name = "Filter"

class DownloadButton(Filter):
    """Download Button.

        A simple download button.

        Args:
            label (str): label of the button.
            url (str): endpoint to call upon click.
            route_func (function): endpoint function.

    """
    def __init__(self, label, url, route_func):
        opts = {
            "url": url,
            "label": label
        }
        super(DownloadButton, self).__init__(
            "DownloadButton",
            opts,
            route_func)

class ConditionalButton(Filter):
    """Conditional Button Group.

        A group of two buttons where the parent choice affects
        the child choice.

        Args:
            labels (list): labels of the button.
            items (dict): choices, children are nested.
            filter_ids (list): list of aliases for the filter.
            defaults (list): default selections.
            filter_style: css for the buttons.

    """
    def __init__(self, labels, items, filter_ids, defaults,
                 filter_style="btn-group"):

        if (len(labels) != 2) or (len(defaults) != 2) or (len(filter_ids) != 2):
            raise Exception("The length of labels, defaults, or filter_ids must be 2")

        item_data = []
        for k, v in list(items.items()):
            item_data.append({"primary": k, "secondary": v})

        opts = {
            "labels": labels,
            "defaults": defaults,
            "aliases": filter_ids,
            "items": item_data,
            "filter_style": filter_style
        }
        super(ConditionalButton, self).__init__("ConditionalSelectButton", opts, None)

class SliderInput(Filter):
    """Simple Slider input.

        A slider bar.

        Args:
            label (str): label of the slider.
            min_val (int): minimum value.
            max_val (int): maximum value.
            filter_id (str): list of aliases for the filter.
            default (int): default selection.
            step (int): size of the step associated with the slider.

    """
    def __init__(self, label, min_val, max_val, filter_id, default, step=1):
        opts = {
            "label": label,
            "min": min_val,
            "max": max_val,
            "step": step,
            "alias": filter_id,
            "default": default

        }
        super(SliderInput, self).__init__("SliderInput", opts, None)

class SelectButton(Filter):
    """Dropdown Button.

        A dropdown selection button based on the React-Bootstrap Dropdown.
        http://react-bootstrap.github.io/components.html#buttons

        Args:
            label (str): label of the button.
            items (dict): choices, children are nested.
            filter_id (str): alias of the filter.
            default (str): default selection.

    """
    def __init__(self, label, items, filter_id, default):
        opts = {
            "label": label,
            "items": items,
            "alias": filter_id,
            "default": default
        }
        super(SelectButton, self).__init__("SelectButton", opts, None)

class ApiButton(Filter):
    """Selections populated by API.

        A dropdown selection button based on the React-Bootstrap Dropdown.
        http://react-bootstrap.github.io/components.html#buttons

        The selections are created by calling an endpoint.

        Args:
            label (str): label of the button.
            url (str): name of endpoint to create.
            filter_id (str): alias of the filter.
            default (str): default selection.
            route_func (function): endpoint function to serve the options.

    """
    def __init__(self, label, url, filter_id, default, route_func):
        opts = {
            "label": label,
            "url": url,
            "alias": filter_id,
            "default": default
        }
        super(ApiButton, self).__init__(
            "ApiButton",
            opts,
            route_func)

class DynamicTextInput(Filter) :
    """Dynamic Text Selection.

        A combination search bar plus drop down inspired by:
        https://realpython.com/blog/python/the-ultimate-flask-front-end-part-2/

        The selections are created by calling an endpoint.

        Args:
            url (str): name of endpoint to create.
            filter_id (str): alias of the filter.
            default (str): default selection.
            route_func (function): endpoint function to serve the options.
            placeholder (str): default text.
            help (str): help text.

    """
    def __init__(self, url, filter_id, default, route_func,
            placeholder="Enter text...", help=""):
        opts = {
            "url": url,
            "help": help,
            "alias": filter_id,
            "default": default,
            "placeholder": placeholder,
            "max": 20
        }
        super(DynamicTextInput, self).__init__(
            "DynamicSearch",
            opts,
            route_func)

class ReactSelect(Filter):
    """ react-select wrapper

        This is a wrapper for the react-select library
        http://jedwatson.github.io/react-select/

        Args:
            url (str): name of endpoint to create.
            filter_id (str): alias of the filter.
            default (str): default selection.
            route_func (function): endpoint function to serve the options.
            placeholder (str): default text.
            extra_args (dict): extra props.
    """
    def __init__(self, url, filter_id, default, route_func,
            placeholder="Enter text...", extra_args={}):
        opts = {
            "url": url,
            "alias": filter_id,
            "default": default,
            "options": {
                "placeholder": placeholder
            }
        }
        for arg in extra_args:
            opts["options"][arg] = extra_args[arg]

        super(ReactSelect, self).__init__("ReactSelect", opts, route_func)

    @staticmethod
    def format_options(opts):
        return {
            "data": [{"label": k, "value": k} for k in opts]
        }



from . import ui
from . import react_template
from . import router

ReactTemplate = react_template.ReactTemplate
ReactComponent = react_template.ReactComponent
ReactRouter = router.ReactRouter
UIComponent = ui.UIComponent
SimpleComponent = ui.SimpleComponent
UILayout = ui.UILayout
register_layouts = ui.register_layouts



import os
from flask import Flask, render_template

DEFAULT_HTML_PARAMS = {
    "page_scripts": ["bundle.js"],
    "base_scripts": [],
    "css": ["main.css"],
    "title": "Pyxley"
}

def create_app(instance_path, static_path, template_path,
    index_params=DEFAULT_HTML_PARAMS, html="index.html"):
    """

        Args:
            instance_path (str): top level
            static_path (str): path to static
    """
    app = Flask(__name__, instance_path=instance_path,
                          instance_relative_config=True,
                          static_folder=static_path,
                          template_folder=template_path)

    def index(name="index"):
        return render_template(html, **index_params)
    app.add_url_rule("/", view_func=index)
    app.add_url_rule("/<string:name>", view_func=index)

    return app

def default_static_path():
    """
        Return the path to the javascript bundle
    """
    fdir = os.path.dirname(__file__)
    return os.path.abspath(os.path.join(fdir, '../assets/'))

def default_template_path():
    """
        Return the path to the index.html
    """
    fdir = os.path.dirname(__file__)
    return os.path.abspath(os.path.join(fdir, '../assets/'))




from .mg import MG
from flask import jsonify, request

class Histogram(MG):
    """Wrapper for MetricsGraphics Histogram plot.

        https://github.com/mozilla/metrics-graphics/wiki/Histogram
        The user must supply a Figure object. When a route_func
        is supplied, the dataframe and other options are ignored
        in favor of the endpoint function.

        Args:
            df (dataframe): input data.
            figure (mg.Figure): metricsgraphics figure object.
            column (str): name of the column to histogram.
            bins (int): number of bins.
            title (str): title of the figure.
            description (str): description of the figure.
            init_params (dict): parameters used to initialize the figure.
            route_func (function): endpoint function.

    """
    def __init__(self, df, figure, column, bins, title="Histogram",
        description="Histogram", init_params={}, route_func=None):

        self.plot_opts = {
            "title": title,
            "description": description,
            "init_params": init_params,
            "bins": bins
        }
        figure.graphics.chart_type("histogram")
        figure.graphics.target("#"+figure.chart_id)
        for k, v in list(figure.get().items()):
            self.plot_opts[k] = v

        if not route_func:
            def get_data():
                args = {}
                for c in init_params:
                    if request.args.get(c):
                        args[c] = request.args[c]
                    else:
                        args[c] = init_params[c]
                return jsonify(Histogram.to_json(
                        self.apply_filters(df, args),
                        column
                    ))
            route_func = get_data

        super(Histogram, self).__init__(figure.chart_id, figure.url,
            self.plot_opts, route_func)

    @staticmethod
    def to_json(df, column):
        """Format output for json response."""
        return {"result": df[column].tolist()}





from ..charts import Chart
from flask import jsonify, request

_BASE_CONFIG = {
    "showLink": False,
    "displaylogo": False,
    "modeBarButtonsToRemove": ["sendDataToCloud"]
}

class PlotlyAPI(Chart):
    """ Base class for Plotly.js API

        This class is used to create charts using the plotly.js api

        To keep this general, this chart does not have a default
        method of transmitting data. Instead the user must supply
        a route_func method.

    """

    def __init__(self, chart_id, url, route_func, init_params={}):

        options = {
            "chartid": chart_id,
            "url": url,
            "params": init_params
        }
        super(PlotlyAPI, self).__init__("PlotlyAPI", options, route_func)

    @staticmethod
    def line_plot(df, xypairs, mode, layout={}, config=_BASE_CONFIG):
        """ basic line plot

            dataframe to json for a line plot

            Args:
                df (pandas.DataFrame): input dataframe
                xypairs (list): list of tuples containing column names
                mode (str): plotly.js mode (e.g. lines)
                layout (dict): layout parameters
                config (dict): config parameters
        """
        if df.empty:
            return {
                "x": [],
                "y": [],
                "mode": mode
            }

        _data = []
        for x, y in xypairs:
            if (x in df.columns) and (y in df.columns):
                _data.append(
                    {
                        "x": df[x].values.tolist(),
                        "y": df[y].values.tolist(),
                        "mode": mode
                    }
                )

        return {
            "data": _data,
            "layout": layout,
            "config": config
            }



from . import npm
from . import webpack
from . import flask_helper

NPM = npm.NPM
Webpack = webpack.Webpack
create_app = flask_helper.create_app
default_static_path = flask_helper.default_static_path
default_template_path = flask_helper.default_template_path



from ..charts import Chart
from flask import jsonify, request


class DataTable(Chart):
    """Wrapper for jquery DataTables.

        This class provides a wrapper for the JQuery DataTables
        component within PyxleyJS. Datatables options can be passed
        through the kwargs.

        Args:
            table_id (str): html element id.
            url (str): name of the endpoint to be created.
            df (dataframe): tabular data to be rendered.
            columns (OrderedDict): columns to display. order is preserved by
                the OrderedDict.
            init_params (dict): parameters used to initialize the table.
            paging (bool): enable paging.
            searching (bool): enable searching.
            sortable (bool): enable sorting.
            classname (str): html classname for css.
            route_func (function): endpoint function. Default is None.


    """
    def __init__(self, table_id, url, df, columns={}, init_params={},
        paging=False, searching=False, sortable=False, classname="display",
        route_func=None, **kwargs):

        opts = {
            "params": init_params,
            "id": table_id,
            "url": url,
            "className": classname,
            "table_options": {
                "paging": paging,
                "searching": searching,
                "bSort": sortable
            }
        }

        for k, v in list(kwargs.items()):
            opts["table_options"][k] = v

        self.columns = columns
        self.confidence = {}
        for k, v in list(self.columns.items()):
            if "confidence" in v:
                self.confidence[k] = v["confidence"]

        if not route_func:
            def get_data():
                args = {}
                for c in init_params:
                    if request.args.get(c):
                        args[c] = request.args[c]
                    else:
                        args[c] = init_params[c]
                return jsonify(DataTable.to_json(
                        self.apply_filters(df, args),
                        self.columns,
                        confidence=self.confidence
                    ))
            route_func = get_data

        super(DataTable, self).__init__("Table", opts, route_func)

    @staticmethod
    def format_row(row, bounds, columns):
        """Formats a single row of the dataframe"""
        for c in columns:
            if c not in row:
                continue

            if "format" in columns[c]:
                row[c] = columns[c]["format"] % row[c]

            if c in bounds:
                b = bounds[c]
                row[c] = [b["min"],row[b["lower"]], row[b["upper"]], b["max"]]

        return row

    @staticmethod
    def to_json(df, columns, confidence={}):
        """Transforms dataframe to properly formatted json response"""
        records = []

        display_cols = list(columns.keys())
        if not display_cols:
            display_cols = list(df.columns)

        bounds = {}
        for c in confidence:
            bounds[c] = {
                "min": df[confidence[c]["lower"]].min(),
                "max": df[confidence[c]["upper"]].max(),
                "lower": confidence[c]["lower"],
                "upper": confidence[c]["upper"]
            }

        labels = {}
        for c in display_cols:
            if "label" in columns[c]:
                labels[c] = columns[c]["label"]
            else:
                labels[c] = c

        for i, row in df.iterrows():
            row_ = DataTable.format_row(row, bounds, columns)
            records.append({labels[c]: row_[c] for c in display_cols})

        return {
            "data": records,
            "columns": [{"data": labels[c]} for c in display_cols]
        }
